unit MyReg;

interface

uses
  Windows, Messages, SysUtils, Classes, StdCtrls, Registry;

type
  TMyReg = class(TRegistry)
  private
    opened: boolean;
  public
    constructor Create(StartRoot: HKEY; FirstKey, KeyName, GroupKey, GroupName: string); overload;
    constructor Create(StartRoot: HKEY; FirstKey, KeyName: string); overload;
    function CheckValue(name: string; valType: TRegDataType): boolean;
    function ReadBin(name: string; var buf; bufSize: integer; var def; defSize: integer): integer;
    function ReadBol(name: string; def: boolean): boolean;
    function ReadYNo(name: string; def: boolean): boolean;
    function ReadCur(name: string; def: currency): currency;
    function ReadDat(name: string; def: TDateTime): TDateTime;
    function ReadDtm(name: string; def: TDateTime): TDateTime;
    function ReadFlt(name: string; def: double): double;
    function ReadInt(name: string; def: integer): integer;
    function ReadStr(name, def: string): string;
    function ReadTme(name: string; def: TDateTime): TDateTime;
    procedure WriteBin(name: string; var buf; bufSize: integer);
    procedure WriteBol(name: string; value: boolean);
    procedure WriteYNo(name: string; value: boolean);
    procedure WriteCur(name: string; value: currency);
    procedure WriteDat(name: string; value: TDateTime);
    procedure WriteDtm(name: string; value: TDateTime);
    procedure WriteFlt(name: string; value: double);
    procedure WriteInt(name: string; value: integer);
    procedure WriteStr(name, value: string);
    procedure WriteTme(name: string; value: TDateTime);
  end;

const
  REG_CU = HKEY_CURRENT_USER;
  REG_LM = HKEY_LOCAL_MACHINE;
  REG_GROUP_KEY = 'Software\Bertel';
  REG_GROUP_NAME = 'Bertel Soft by Michael Berthold';

implementation

constructor TMyReg.Create(StartRoot: HKEY; FirstKey, KeyName, GroupKey, GroupName: string);
begin
  Inherited Create;
  opened := FALSE;
  RootKey := StartRoot;
  if OpenKey(GroupKey, TRUE) then begin
    if ReadString('') = '' then
      WriteString('', GroupName);
    opened := OpenKey(FirstKey, TRUE);
    if opened then
      if ReadString('') = '' then
        WriteString('', KeyName);
  end;
end;

constructor TMyReg.Create(StartRoot: HKEY; FirstKey: string; KeyName: string);
begin
  Create(StartRoot, FirstKey, KeyName, REG_GROUP_KEY, REG_GROUP_NAME);
end;

function TMyReg.CheckValue(name: string; valType: TRegDataType): boolean;
begin
  CheckValue := FALSE;
  if ValueExists(name) then
    CheckValue := (GetDataType(name) = valType);
end;

{ Read Operations }

function TMyReg.ReadBin(name: string; var buf; bufSize: integer; var def; defSize: integer): integer;
begin
  if CheckValue(name, rdBinary) then
    ReadBin := ReadBinaryData(name, buf, bufSize)
  else begin
    if (defSize > bufSize) then defSize := bufSize;
    CopyMemory(@buf, @def, defSize);
    ReadBin := 0;
  end;
end;

function TMyReg.ReadBol(name: string; def: boolean): boolean;
begin
  if CheckValue(name, rdInteger) then
    ReadBol := ReadBool(name)
  else ReadBol := def;
end;

function TMyReg.ReadYNo(name: string; def: boolean): boolean;
var      S: String;
begin
  if CheckValue(name, rdString) then begin
    S := ReadString(name);
    Result := (CompareText(S, 'yes') = 0);
  end else Result := def;
end;

function TMyReg.ReadCur(name: string; def: currency): currency;
begin
  if CheckValue(name, rdBinary) then
    ReadCur := ReadCurrency(name)
  else ReadCur := def;
end;

function TMyReg.ReadDat(name: string; def: TDateTime): TDateTime;
begin
  if CheckValue(name, rdBinary) then
    ReadDat := ReadDate(name)
  else ReadDat := def;
end;

function TMyReg.ReadDtm(name: string; def: TDateTime): TDateTime;
begin
  if CheckValue(name, rdBinary) then
    ReadDtm := ReadDateTime(name)
  else ReadDtm := def;
end;

function TMyReg.ReadFlt(name: string; def: double): double;
begin
  if CheckValue(name, rdBinary) then
    ReadFlt := ReadFloat(name)
  else ReadFlt := def;
end;

function TMyReg.ReadInt(name: string; def: integer): integer;
begin
  if CheckValue(name, rdInteger) then
    ReadInt := ReadInteger(name)
  else ReadInt := def;
end;

function TMyReg.ReadStr(name, def: string): string;
begin
  if CheckValue(name, rdString) then
    ReadStr := ReadString(name)
  else ReadStr := def;
end;

function TMyReg.ReadTme(name: string; def: TDateTime): TDateTime;
begin
  if CheckValue(name, rdBinary) then
    ReadTme := ReadTime(name)
  else ReadTme := def;
end;

{ Write Operations }

procedure TMyReg.WriteBin(name: string; var buf; bufSize: integer);
begin
  if CheckValue(name, rdBinary) then DeleteValue(name);
  WriteBinaryData(name, buf, bufSize);
end;

procedure TMyReg.WriteBol(name: string; value: boolean);
begin
  if CheckValue(name, rdInteger) then DeleteValue(name);
  WriteBool(name, value);
end;

procedure TMyReg.WriteYNo(name: string; value: boolean);
var       S: String;
begin
  if value then S := 'yes' else S := 'no';
  if CheckValue(name, rdString) then DeleteValue(name);
  WriteString(name, S);
end;

procedure TMyReg.WriteCur(name: string; value: currency);
begin
  if CheckValue(name, rdBinary) then DeleteValue(name);
  WriteCurrency(name, value);
end;

procedure TMyReg.WriteDat(name: string; value: TDateTime);
begin
  if CheckValue(name, rdBinary) then DeleteValue(name);
  WriteDate(name, value);
end;

procedure TMyReg.WriteDtm(name: string; value: TDateTime);
begin
  if CheckValue(name, rdBinary) then DeleteValue(name);
  WriteDateTime(name, value);
end;

procedure TMyReg.WriteFlt(name: string; value: double);
begin
  if CheckValue(name, rdBinary) then DeleteValue(name);
  WriteFloat(name, value);
end;

procedure TMyReg.WriteInt(name: string; value: integer);
begin
  if CheckValue(name, rdInteger) then DeleteValue(name);
  WriteInteger(name, value);
end;

procedure TMyReg.WriteStr(name, value: string);
begin
  if CheckValue(name, rdString) then DeleteValue(name);
  WriteString(name, value);
end;

procedure TMyReg.WriteTme(name: string; value: TDateTime);
begin
  if CheckValue(name, rdBinary) then DeleteValue(name);
  WriteTime(name, value);
end;

end.
