unit UFormAccount;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IniFiles, UOptions;

type
  TFormAccount = class(TForm)
    ButtonOK: TButton;
    ButtonCancel: TButton;
    GroupBoxData: TGroupBox;
    LabelServer: TLabel;
    LabelPort: TLabel;
    LabelUser: TLabel;
    LabelPassword: TLabel;
    EditServer: TEdit;
    EditPort: TEdit;
    EditUser: TEdit;
    EditPassword: TEdit;
    GroupBoxAccounts: TGroupBox;
    ButtonNew: TButton;
    ButtonSave: TButton;
    ButtonDel: TButton;
    CheckBoxSavePass: TCheckBox;
    ListBoxAccounts: TListBox;
    ButtonExport: TButton;
    ButtonImport: TButton;
    OpenDialogImport: TOpenDialog;
    SaveDialogExport: TSaveDialog;
    EditTitle: TEdit;
    LabelTitle: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ListBoxAccountsClick(Sender: TObject);
    procedure ButtonNewClick(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
    procedure ButtonDelClick(Sender: TObject);
    procedure ButtonExportClick(Sender: TObject);
    procedure ButtonImportClick(Sender: TObject);
    procedure ButtonOKClick(Sender: TObject);
  private
    Options: TPOP3DAOptions;
    function MsgBox(MsgStr: String; MsgFlags: Cardinal): Integer;
    function ValidateInput: Boolean;
  public
    constructor Create(AOwner: TComponent; AOptions: TPOP3DAOptions); reintroduce; virtual;
  end;

var
  FormAccount: TFormAccount;

implementation

{$R *.dfm}

function TFormAccount.MsgBox(MsgStr: String; MsgFlags: Cardinal): Integer;
begin
  Result := MessageBox(Handle, PChar(MsgStr), PChar(Caption), MsgFlags);
end;

constructor TFormAccount.Create(AOwner: TComponent; AOptions: TPOP3DAOptions);
begin
  Options := AOptions;
  Inherited Create(AOwner);
end;

procedure TFormAccount.FormCreate(Sender: TObject);
var
  iAccounts: Integer;
begin
  ListBoxAccounts.Clear;
  if Options.AccountCount > 0 then begin
    for iAccounts := 0 to Options.AccountCount - 1 do
      ListBoxAccounts.Items.AddObject(Options.Account[iAccounts].Title, Options.Account[iAccounts]);
    ListBoxAccounts.ItemIndex := 0;
    ListBoxAccountsClick(ListBoxAccounts);
  end;
end;

procedure TFormAccount.ListBoxAccountsClick(Sender: TObject);
var
  Account: TAccountData;
begin
  if ListBoxAccounts.ItemIndex >= 0 then begin
    Account := ListBoxAccounts.Items.Objects[ListBoxAccounts.ItemIndex] as TAccountData;
    if Account <> nil then begin
      EditTitle.Text := Account.Title;
      EditServer.Text := Account.Server;
      EditPort.Text := Account.PortStr;
      EditUser.Text := Account.User;
      if Account.SavePassword then begin
        EditPassword.Text := Account.Password;
        CheckBoxSavePass.Checked := True;
      end else begin
        EditPassword.Text := '';
        CheckBoxSavePass.Checked := False;
      end;
    end;
  end;
end;

procedure TFormAccount.ButtonNewClick(Sender: TObject);
begin
  ListBoxAccounts.ItemIndex := -1;
  EditTitle.Text := '';
  EditServer.Text := '';
  EditPort.Text := '110';
  EditUser.Text := '';
  EditPassword.Text := '';
  CheckBoxSavePass.Checked := False;
end;

function TFormAccount.ValidateInput: Boolean;
  procedure Warning(MsgText: String);
  begin
    MsgBox(MsgText, MB_ICONEXCLAMATION);
  end;
begin
  Result := False;
  if EditTitle.Text = '' then begin
    Warning('You must enter an account title!');
    EditTitle.SetFocus;
  end else if EditServer.Text = '' then begin
    Warning('You must enter a server name or ip address, i.e. pop.gmx.net');
    EditServer.SetFocus;
  end else if StrToIntDef(EditPort.Text, -1) < 0 then begin
    Warning('You must enter a valid port number, i.e. 110');
    EditPort.SetFocus;
  end else if EditUser.Text = '' then begin
    Warning('You must enter a user name, i.e. your.name@gmx.net or YourName234');
    EditUser.SetFocus;
  end else Result := True;
end;

procedure TFormAccount.ButtonSaveClick(Sender: TObject);
var
  Account: TAccountData;
begin
  if ValidateInput then begin
    if ListBoxAccounts.ItemIndex >= 0 then begin
      Account := ListBoxAccounts.Items.Objects[ListBoxAccounts.ItemIndex] as TAccountData;
      Account.Title := EditTitle.Text;
    end else begin
      Account := Options.AccountAdd;
      ListBoxAccounts.Items.AddObject(EditTitle.Text, Account);
    end;
    if Account <> nil then begin
      Account.Title := EditTitle.Text;
      Account.Server := EditServer.Text;
      Account.PortStr := EditPort.Text;
      Account.User := EditUser.Text;
      if CheckBoxSavePass.Checked then begin
        Account.Password := EditPassword.Text;
        Account.SavePassword := True;
      end else begin
        Account.Password := '';
        Account.SavePassword := False;
      end;
    end;
  end;
end;

procedure TFormAccount.ButtonDelClick(Sender: TObject);
var
  Account: TAccountData;
begin
  if ListBoxAccounts.ItemIndex >= 0 then begin
    Account := ListBoxAccounts.Items.Objects[ListBoxAccounts.ItemIndex] as TAccountData;
    if Account <> nil then begin
      Options.AccountDel(Account.Index);
      ListBoxAccounts.DeleteSelected;
      ButtonNewClick(ButtonNew);
    end;
  end;
end;

procedure TFormAccount.ButtonExportClick(Sender: TObject);
begin
  if SaveDialogExport.Execute then Options.Export(SaveDialogExport.FileName);
end;

procedure TFormAccount.ButtonImportClick(Sender: TObject);
begin
  if OpenDialogImport.Execute then begin
    Options.Import(OpenDialogImport.FileName);
    FormCreate(Sender);
  end;
end;

procedure TFormAccount.ButtonOKClick(Sender: TObject);
begin
  if ValidateInput then ModalResult := mrOK;
end;

end.
