unit UFormMail;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, UFormPOP3DA, IdPOP3, IdMessage, IdExceptionCore,
  IdText, IdAttachment, IdMessageParts, ExtCtrls, Menus, ShellAPI;

type
  TFormMail = class(TForm)
    GroupBoxData: TGroupBox;
    LabelFrom: TLabel;
    LabelTo: TLabel;
    LabelCC: TLabel;
    LabelSubject: TLabel;
    GroupBoxText: TGroupBox;
    EditFrom: TEdit;
    EditTo: TEdit;
    EditCC: TEdit;
    EditSubject: TEdit;
    LabelSent: TLabel;
    EditSent: TEdit;
    MemoText: TMemo;
    StatusBar: TStatusBar;
    Timer: TTimer;
    ListBoxTextParts: TListBox;
    ButtonSave: TButton;
    ButtonOpen: TButton;
    LabelTextParts: TLabel;
    Label1: TLabel;
    ListBoxAttachments: TListBox;
    SaveDialogAttachment: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PopupItemSaveClick(Sender: TObject);
    procedure ListBoxTextPartsClick(Sender: TObject);
    procedure ListBoxAttachmentsClick(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
  private
    MsgNum: Integer;
    IdMessage: TIdMessage;
    function MsgBox(MsgStr: String; MsgFlags: Cardinal): Integer;
  public
    constructor Create(AOwner: TComponent; AMsgNum: Integer); reintroduce; virtual;
  end;

var
  FormMail: TFormMail;

implementation

{$R *.dfm}

function TFormMail.MsgBox(MsgStr: String; MsgFlags: Cardinal): Integer;
begin
  Result := MessageBox(Handle, PChar(MsgStr), PChar(Caption), MsgFlags);
end;

constructor TFormMail.Create(AOwner: TComponent; AMsgNum: Integer);
begin
  MsgNum := AMsgNum;
  Inherited Create(AOwner);
end;

procedure TFormMail.FormCreate(Sender: TObject);
begin
  IdMessage := TIdMessage.Create(Self);
  Timer.Enabled := True;
end;

procedure TFormMail.FormDestroy(Sender: TObject);
begin
  IdMessage.Free;
end;

procedure TFormMail.TimerTimer(Sender: TObject);
var
  Error: Boolean;
  iMsg, TextPartNum, AttachmentNum, MsgSize: Integer;
  MsgPart: TIdMessagePart;
begin
  Timer.Enabled := False;
  Error := False;
  try
    StatusBar.SimpleText := 'Please wait while retrieving message...';
    FormPOP3DA.ConnectIdPOP3;
    MsgSize := FormPOP3DA.IdPOP3.RetrieveMsgSize(MsgNum);
    if (MsgSize <= 100000) or (MsgBox('The message is bigger than 100kB. Do you really want to download it?', MB_ICONQUESTION + MB_YESNO) = IDYES) then begin
      if FormPOP3DA.IdPOP3.Retrieve(MsgNum, IdMessage) then begin
        EditFrom.Text := IdMessage.FromList.EMailAddresses;
        EditTo.Text := IdMessage.Recipients.EMailAddresses;
        EditCC.Text := IdMessage.CCList.EMailAddresses;
        EditSubject.Text := IdMessage.Subject;
        EditSent.Text := FormatDateTime(LongDateFormat + ', ' + LongTimeFormat, IdMessage.Date);
        MemoText.Text := IdMessage.Body.Text;
        TextPartNum := 0;
        AttachmentNum := 0;
        for iMsg := 0 to IdMessage.MessageParts.Count - 1 do begin
          MsgPart := IdMessage.MessageParts[iMsg];
          if MsgPart is TIdText then begin
            Inc(TextPartNum);
            ListBoxTextParts.Items.AddObject('Text '+ IntToStr(TextPartNum), TObject(iMsg));
          end else if MsgPart is TIdAttachment then begin
            Inc(AttachmentNum);
            ListBoxAttachments.Items.AddObject('Attachment '+ IntToStr(AttachmentNum), TObject(iMsg));
          end;
        end;
      end;
    end else Close;
  except
    on E: Exception do begin
      StatusBar.SimpleText := 'Error retrieving Msg: ' + E.Message;
      Error := True;
    end;
  end;
  FormPOP3DA.DisconnectIdPOP3;
  if not Error then StatusBar.SimpleText := 'Ready.';
end;

procedure TFormMail.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if FormPOP3DA.IdPOP3.Connected then
    try
      FormPOP3DA.IdPOP3.Disconnect;
    except
      on E: Exception do StatusBar.SimpleText := 'Error disconnecting: ' + E.Message;
    end;
end;

procedure TFormMail.PopupItemSaveClick(Sender: TObject);
begin
  //
end;

procedure TFormMail.ListBoxTextPartsClick(Sender: TObject);
var
  MsgPart: TIdMessagePart;
begin
  if ListBoxTextParts.ItemIndex = 0 then begin
    MemoText.Text := IdMessage.Body.Text;
  end else if ListBoxTextParts.ItemIndex > 0 then begin
    MsgPart := IdMessage.MessageParts[Integer(ListBoxTextParts.Items.Objects[ListBoxTextParts.ItemIndex])];
    if MsgPart is TIdText then MemoText.Text := TIdText(MsgPart).Body.Text;
    ListBoxAttachments.ItemIndex := -1;
  end;
end;

procedure TFormMail.ListBoxAttachmentsClick(Sender: TObject);
var
  MsgPart: TIdMessagePart;
begin
  if ListBoxAttachments.ItemIndex >= 0 then begin
    MsgPart := IdMessage.MessageParts[Integer(ListBoxAttachments.Items.Objects[ListBoxAttachments.ItemIndex])];
    if MsgPart is TIdAttachment then begin
      MemoText.Text := 'SYSTEM: This Part is an Attachment.'#13#10 +
        'Filename: ' + TIdAttachment(MsgPart).FileName + #13#10 +
        'Content type: ' + TIdAttachment(MsgPart).ContentType;
      ButtonSave.Enabled := True;
      ButtonOpen.Enabled := True;
    end;
    ListBoxTextParts.ItemIndex := -1;
  end;
end;

procedure TFormMail.ButtonSaveClick(Sender: TObject);
  function GetTempDir: String;
  var
    TempDir: Array[0..4095] of Char;
  begin
    if GetTempPath(4096, TempDir) > 0 then Result := IncludeTrailingBackslash(StrPas(TempDir))
    else Result := '';
  end;
var
  MsgPart: TIdMessagePart;
  TempName: String;
begin
  if ListBoxAttachments.ItemIndex >= 0 then begin
    MsgPart := IdMessage.MessageParts[Integer(ListBoxAttachments.Items.Objects[ListBoxAttachments.ItemIndex])];
    if MsgPart is TIdAttachment then begin
      if Sender = ButtonSave then begin
        SaveDialogAttachment.FileName := TIdAttachment(MsgPart).FileName;
        if SaveDialogAttachment.Execute then
          try
            TIdAttachment(MsgPart).SaveToFile(SaveDialogAttachment.FileName);
          except
            on E:Exception do MsgBox('Error saving file: ' + E.Message, MB_ICONERROR);
          end;
      end else if Sender = ButtonOpen then begin
        TempName := GetTempDir;
        if TIdAttachment(MsgPart).FileName > '' then TempName := TempName + TIdAttachment(MsgPart).FileName
        else TempName := TempName + '45_gzef-da~ew.dat';
          try
            TIdAttachment(MsgPart).SaveToFile(TempName);
            ShellExecute(Handle, 'open', PChar(TempName), nil, nil, SW_SHOW);
          except
            on E:Exception do MsgBox('Error saving temp file (' + TempName + '): ' + E.Message, MB_ICONERROR);
          end;
      end;
    end;
  end;
end;

end.
