unit UFormPOP3DA;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, IdPOP3, Menus, ImgList, IdBaseComponent,
  IdComponent, IdTCPConnection, IdTCPClient, IdExplicitTLSClientServerBase,
  IdMessageClient, ExtCtrls, UOptions;

type
  TFormPOP3DA = class(TForm)
    IdPOP3: TIdPOP3;
    GroupBoxMails: TGroupBox;
    ListViewMails: TListView;
    GroupBoxSystem: TGroupBox;
    MemoInfo: TMemo;
    GroupBoxControl: TGroupBox;
    ButtonAbort: TButton;
    ButtonDelete: TButton;
    StatusBar: TStatusBar;
    PopupMenuMails: TPopupMenu;
    PopupItemCheck: TMenuItem;
    PopupItemUncheck: TMenuItem;
    N1: TMenuItem;
    PopupItemCheckAll: TMenuItem;
    PopupItemUncheckAll: TMenuItem;
    GroupBoxHeader: TGroupBox;
    MemoHeader: TMemo;
    ImageList: TImageList;
    N2: TMenuItem;
    PopupItemView: TMenuItem;
    ButtonAccount: TButton;
    ButtonInfo: TButton;
    Timer: TTimer;
    ButtonGetMail: TButton;
    ButtonHelp: TButton;
    ButtonClose: TButton;
    TimerConnect: TTimer;
    procedure ButtonGetMailClick(Sender: TObject);
    procedure ButtonAbortClick(Sender: TObject);
    procedure ButtonDeleteClick(Sender: TObject);
    procedure IdPOP3Disconnected(Sender: TObject);
    procedure IdPOP3Status(ASender: TObject; const AStatus: TIdStatus; const AStatusText: String);
    procedure ListViewMailsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure PopupItemCheckClick(Sender: TObject);
    procedure ListViewMailsDblClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListViewMailsColumnClick(Sender: TObject; Column: TListColumn);
    procedure ListViewMailsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure ButtonAccountClick(Sender: TObject);
    procedure PopupMenuMailsPopup(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure ButtonInfoClick(Sender: TObject);
    procedure ButtonHelpClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure TimerConnectTimer(Sender: TObject);
  private
    AbortAction: Boolean;
    Options: TPOP3DAOptions;
    function MsgBox(MsgStr: String; MsgFlags: Cardinal): Integer;
  public
    procedure AddSystemInfo(SysInfo: String);
    function ConnectIdPOP3: Boolean;
    procedure DisconnectIdPOP3;
  end;

var
  FormPOP3DA: TFormPOP3DA;

implementation

uses
  IdMessage, IdExceptionCore, UFormMail, UFormAccount, UFormAbout, UListView,
  ShellAPI;

{$R *.dfm}

function TFormPOP3DA.MsgBox(MsgStr: String; MsgFlags: Cardinal): Integer;
begin
  Result := MessageBox(Handle, PChar(MsgStr), PChar(Caption), MsgFlags);
end;

procedure TFormPOP3DA.AddSystemInfo(SysInfo: String);
begin
  MemoInfo.Lines.Add(FormatDateTime('hh:nn:ss.zzz', Time) + ': ' + StringReplace(SysInfo, #13#10, '', [rfReplaceAll]));
end;

function TFormPOP3DA.ConnectIdPOP3: Boolean;
begin
  // Im Falle, dass der Disconnect-Timer luft, wird dieser erstmal deaktiviert.
  TimerConnect.Enabled := False;
  Result := False;
  try
    if not IdPOP3.Connected then begin
      IdPOP3.Connect;
      AddSystemInfo('Connected to ' + IdPOP3.Host);
      IdPOP3.Login;
      AddSystemInfo('User ' + IdPOP3.Username + ' logged in');
      Result := True;
    end;
  except
    on E: Exception do begin
      AddSystemInfo('Error connecting to or authenticating at ' + IdPOP3.Host + '. ' + E.Message);
    end;
  end;
end;

procedure TFormPOP3DA.DisconnectIdPOP3;
begin
  // Es wird nur der Timer ausgelst, der in 2 Minuten des Disconntect durchfhrt.
  if IdPOP3.Connected then TimerConnect.Enabled := True;
end;

procedure TFormPOP3DA.ButtonGetMailClick(Sender: TObject);
  function AddListItem(IdMessage: TIdMessage; MsgSize, MsgNum: Integer): TListItem;
  var
    Header: String;
  begin
    Result := ListViewMails.Items.Add;
    with Result do begin
      Data := Pointer(MsgNum);
      Caption := IdMessage.FromList.EMailAddresses;
      SubItems.Add(IdMessage.Recipients.EMailAddresses);
      SubItems.Add(IdMessage.Subject);
      SubItems.AddObject(DateTimeToStr(IdMessage.Date), TObject(DateTimeToFileDate(IdMessage.Date)));
      SubItems.AddObject(IntToStr(MsgSize), TObject(MsgSize));
      SubItems.Add(IdMessage.MsgId);
      Header := IdMessage.Headers.Text;
      if IdMessage.ExtraHeaders.Count > 0 then
        Header := Header + #13#10#13#10'**** SYSTEM: Extra Headers ****'#13#10#13#10 + IdMessage.ExtraHeaders.Text;
      SubItems.Add(Header);
      ImageIndex := 0;
    end;
  end;
var
  MsgNum, MsgCount, MsgSize, ErrCount: Integer;
  IdMessage: TIdMessage;
begin
  ButtonAccount.Enabled := False;
  ButtonGetMail.Enabled := False;
  ButtonAbort.Enabled := True;
  ButtonDelete.Enabled := False;
  AbortAction := False;
  if ConnectIdPOP3 then
    try
      MsgCount := IdPOP3.CheckMessages;
      AddSystemInfo('Inbox contains ' + IntToStr(MsgCount) + ' messages');
      ListViewMails.Items.BeginUpdate;
      ListViewMails.Items.Clear;
      IdMessage := TIdMessage.Create(Self);
      ErrCount := 0;
      MsgNum := 1;
      while MsgNum <= MsgCount do begin
        if AbortAction then Break;
        try
          StatusBar.SimpleText := 'Retrieving Message ' + IntToStr(MsgNum);
          MsgSize := IdPOP3.RetrieveMsgSize(MsgNum);
          if IdPOP3.RetrieveHeader(MsgNum, IdMessage) then AddListItem(IdMessage, MsgSize, MsgNum);
          ErrCount := 0;
          Inc(MsgNum);
        except
          on E: Exception do begin
            // Bei Read-Timeouts gibt es einen zweiten Versuch
            if not ((E is EIdReadTimeout) and (ErrCount = 0)) then begin
              AddSystemInfo('Error retrieving Message ' + IntToStr(MsgNum) + ' Header. ' + E.Message);
              Inc(MsgNum);
            end else Inc(ErrCount);
            // Wenn die Connection weg ist, immer abbrechen
            if not IdPOP3.Connected then Break;
          end;
        end;
        Application.ProcessMessages;
      end;
      IdMessage.Free;
      ListViewMails.Items.EndUpdate;
      ListViewMails.AlphaSort;
    except
      on E: Exception do begin
        AddSystemInfo('Error Retrieving message count. ' + E.Message);
      end;
    end;
  DisconnectIdPOP3;
  StatusBar.SimpleText := 'Ready.';
  ButtonAccount.Enabled := True;
  ButtonGetMail.Enabled := True;
  ButtonAbort.Enabled := False;
  ButtonDelete.Enabled := True;
end;

procedure TFormPOP3DA.ButtonAbortClick(Sender: TObject);
begin
  AbortAction := True;
end;

procedure TFormPOP3DA.ButtonDeleteClick(Sender: TObject);
var
  iItem, CheckedCount: Integer;
begin
  CheckedCount := 0;
  for iItem := 0 to ListViewMails.Items.Count - 1 do if ListViewMails.Items[iItem].Checked then Inc(CheckedCount);
  if CheckedCount > 0 then begin
    if MsgBox('Do you really want to delete the ' + IntToStr(CheckedCount) + ' checked messages?', MB_ICONQUESTION + MB_YESNO) = IDYES then begin
      ButtonAccount.Enabled := False;
      ButtonGetMail.Enabled := False;
      ButtonAbort.Enabled := True;
      ButtonDelete.Enabled := False;
      AbortAction := False;
      if ConnectIdPOP3 then begin
        for iItem := 0 to ListViewMails.Items.Count - 1 do begin
          if AbortAction then Break;
          if ListViewMails.Items[iItem].Checked then begin
            try
              StatusBar.SimpleText := 'Deleting Message ' + IntToStr(Integer(ListViewMails.Items[iItem].Data));
              IdPOP3.Delete(Integer(ListViewMails.Items[iItem].Data));
            except
              on E: Exception do begin
                AddSystemInfo('Error deleting Message ' + IntToStr(Integer(ListViewMails.Items[iItem].Data)) + '. ' + E.Message);
                if not IdPOP3.Connected then Break;
              end;
            end;
          end;
        end;
      end;
      DisconnectIdPOP3;
      StatusBar.SimpleText := 'Ready.';
      ButtonAccount.Enabled := True;
      ButtonGetMail.Enabled := True;
      ButtonAbort.Enabled := False;
      ButtonDelete.Enabled := True;
    end;
  end else MsgBox('There are no messages checked.', MB_ICONINFORMATION);
end;

procedure TFormPOP3DA.IdPOP3Disconnected(Sender: TObject);
begin
  AddSystemInfo('Disconnected.');
end;

procedure TFormPOP3DA.IdPOP3Status(ASender: TObject; const AStatus: TIdStatus; const AStatusText: String);
begin
  AddSystemInfo('Status: ' + AStatusText);
end;

procedure TFormPOP3DA.ListViewMailsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  if Selected then begin
    if Item.SubItems.Count > 0 then MemoHeader.Text := Item.SubItems[Item.SubItems.Count - 1];
  end else begin
    MemoHeader.Clear;
  end;
end;

procedure TFormPOP3DA.PopupItemCheckClick(Sender: TObject);
var
  iSel: Integer;
begin
  if Sender is TMenuItem then
    case TMenuItem(Sender).Tag of
      1: for iSel := 0 to ListViewMails.Items.Count - 1 do if ListViewMails.Items[iSel].Selected then ListViewMails.Items[iSel].Checked := True;
      2: for iSel := 0 to ListViewMails.Items.Count - 1 do if ListViewMails.Items[iSel].Selected then ListViewMails.Items[iSel].Checked := False;
      3: for iSel := 0 to ListViewMails.Items.Count - 1 do ListViewMails.Items[iSel].Checked := True;
      4: for iSel := 0 to ListViewMails.Items.Count - 1 do ListViewMails.Items[iSel].Checked := False;
      10: if Assigned(ListViewMails.Selected) then
        with TFormMail.Create(Self, Integer(ListViewMails.Selected.Data)) do
          try
            ShowModal;
          finally
            Free;
          end;
    end;
end;

procedure TFormPOP3DA.ListViewMailsDblClick(Sender: TObject);
begin
  PopupItemCheckClick(PopupItemView);
end;

procedure TFormPOP3DA.FormCreate(Sender: TObject);
begin
  Options := TPOP3DAOptions.Create;
  Options.Read;
  ListViewInit(ListViewMails, 0, 1, 0);
  IdPOP3.ConnectTimeout := 10000;
  IdPOP3.ReadTimeout := 10000;
  Timer.Enabled := True;
end;

procedure TFormPOP3DA.FormDestroy(Sender: TObject);
begin
  // Falls die Verbindung noch besteht, ist es jetzt hchste Zeit frs Ende
  TimerConnectTimer(TimerConnect);
  Options.Write;
  Options.Free;
end;

procedure TFormPOP3DA.ListViewMailsColumnClick(Sender: TObject; Column: TListColumn);
begin
  ListViewColumnClick(ListViewMails, Column);
end;

procedure TFormPOP3DA.ListViewMailsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  ListViewCompare(ListViewMails, Item1, Item2, Data, Compare);
end;

procedure TFormPOP3DA.ButtonAccountClick(Sender: TObject);
begin
  with TFormAccount.Create(Self, Options) do
    try
      if ShowModal = mrOK then begin
        TimerConnectTimer(nil); // erstmal die Leitung trennen
        IdPOP3.Host := EditServer.Text;
        IdPOP3.Port := StrToIntDef(EditPort.Text, 110);
        IdPOP3.Username := EditUser.Text;
        IdPOP3.Password := EditPassword.Text;
        ButtonDelete.Enabled := True;
        ButtonGetMail.Enabled := True;
      end;
    finally
      Free;
    end;
end;

procedure TFormPOP3DA.PopupMenuMailsPopup(Sender: TObject);
var
  HasListItems, HasSelectedListItems: Boolean;
begin
  HasListItems := ListViewMails.Items.Count > 0;
  if HasListItems then HasSelectedListItems := ListViewMails.SelCount > 0
  else HasSelectedListItems := False;
  PopupItemCheck.Enabled := HasSelectedListItems;
  PopupItemUncheck.Enabled := HasSelectedListItems;
  PopupItemCheckAll.Enabled := HasListItems;
  PopupItemUncheckAll.Enabled := HasListItems;
  PopupItemView.Enabled := HasSelectedListItems and ButtonGetMail.Enabled;
end;

procedure TFormPOP3DA.TimerTimer(Sender: TObject);
begin
  Timer.Enabled := False;
  ButtonAccountClick(nil);
end;

procedure TFormPOP3DA.ButtonInfoClick(Sender: TObject);
begin
  with TFormAbout.Create(Self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TFormPOP3DA.ButtonHelpClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', 'http://www.bertel.de/software/pop3da/help.html', nil, nil, SW_SHOW);
end;

procedure TFormPOP3DA.ButtonCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFormPOP3DA.TimerConnectTimer(Sender: TObject);
begin
  TimerConnect.Enabled := False;
  if IdPOP3.Connected then IdPOP3.Disconnect;
end;

end.
