unit UListView;

interface

uses
  ComCtrls;

type
  TListViewTagValue = record
    SortedColumn, SortDirection, DefaultIconIndex, Reserved: Byte;
  end;
  PTListViewTagValue = ^TListViewTagValue;

const
  cLVSD_Ascending: Byte = 1;
  cLVSD_Descending: Byte = 2;

function GenerateListViewTagValue(SortedColumn: Byte = 0; SortDirection: Byte = 1; DefaultIconIndex: Byte = 3): Longint;
procedure ListViewInit(LVControl: TListView; SortedColumn: Byte = 0; SortDirection: Byte = 1; DefaultIconIndex: Byte = 3);

procedure ListViewColumnSetImagesRight(LVControl: TListView);
procedure ListViewColumnClick(LVControl: TListView; Column: TListColumn);
procedure ListViewCompare(LVControl: TListView; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);

implementation

uses
  CommCtrl, Windows, SysUtils;

function GenerateListViewTagValue(SortedColumn: Byte = 0; SortDirection: Byte = 1; DefaultIconIndex: Byte = 3): Longint;
var
  TagValue: TListViewTagValue;
begin
  TagValue.SortedColumn := SortedColumn;
  TagValue.SortDirection := SortDirection;
  TagValue.DefaultIconIndex := DefaultIconIndex;
  TagValue.Reserved := 0;
  Result := Longint(TagValue);
end;

procedure ListViewInit(LVControl: TListView; SortedColumn: Byte = 0; SortDirection: Byte = 1; DefaultIconIndex: Byte = 3);
var
  LVCol: TLVColumn;
  iLVC: Cardinal;
begin
  for iLVC := 0 to LVControl.Columns.Count - 1 do LVControl.Columns[iLVC].ImageRight := True;
  if SortDirection = cLVSD_Ascending then begin
    SortDirection := cLVSD_Descending;
  end else SortDirection := cLVSD_Ascending;
  LVControl.Tag := GenerateListViewTagValue(SortedColumn, SortDirection, DefaultIconIndex);
  ListViewColumnClick(LVControl, LVControl.Columns[SortedColumn]);
end;

procedure ListViewColumnSetImagesRight(LVControl: TListView);
var
  LVCol: TLVColumn;
  iLVC: Cardinal;
begin
  for iLVC := 0 to LVControl.Columns.Count - 1 do
    with LVControl.Columns[iLVC] do begin
      if ImageIndex in [0..2] then begin
        LVCol.mask := LVCF_FMT or LVCF_IMAGE or LVCF_TEXT;
        LVCol.fmt := Ord(Alignment) + LVCFMT_BITMAP_ON_RIGHT or LVCFMT_IMAGE or LVCFMT_COL_HAS_IMAGES;
        LVCol.pszText := PChar(Caption);
        LVCol.iImage := ImageIndex;
        SendMessage(LVControl.Handle, LVM_SETCOLUMN, iLVC, Cardinal(@LVCol));
      end;
    end;
end;

procedure ListViewColumnClick(LVControl: TListView; Column: TListColumn);
var
  TagValue: TListViewTagValue;
begin
  TagValue := TListViewTagValue(LVControl.Tag);
  if TagValue.SortedColumn = Column.Index then begin
    if TagValue.SortDirection = cLVSD_Ascending then begin
      TagValue.SortDirection := cLVSD_Descending;
      Column.ImageIndex := 1;
    end else begin
      TagValue.SortDirection := cLVSD_Ascending;
      Column.ImageIndex := 2;
    end;
  end else begin
    // Sortierrichtungssymbol bei vorhergehender Spalte entfernen
    LVControl.Column[TagValue.SortedColumn].ImageIndex := -1;
    TagValue.SortedColumn := Column.Index;
    TagValue.SortDirection := cLVSD_Ascending;
    Column.ImageIndex := 2;
  end;
  LVControl.Tag := Longint(TagValue);
  LVControl.AlphaSort;
  //ListViewColumnSetImagesRight(LVControl);
end;

procedure ListViewCompare(LVControl: TListView; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  TagValue: TListViewTagValue;
  SortByData: Boolean;
  S1, S2: String;
  D1, D2: Integer;
begin
  TagValue := TListViewTagValue(LVControl.Tag);
  try
    SortByData := (LVControl.Columns[TagValue.SortedColumn].Tag > 0);
    if TagValue.SortedColumn = 0 then begin
      if SortByData then begin
        D1 := Integer(Item1.Data);
        D2 := Integer(Item2.Data);
      end else begin
        S1 := Item1.Caption;
        S2 := Item2.Caption;
      end;
    end else begin
      try
        if SortByData then begin
          D1 := Integer(Item1.SubItems.Objects[TagValue.SortedColumn - 1]);
          D2 := Integer(Item2.SubItems.Objects[TagValue.SortedColumn - 1]);
        end else begin
          S1 := Item1.SubItems.Strings[TagValue.SortedColumn - 1];
          S2 := Item2.SubItems.Strings[TagValue.SortedColumn - 1];
        end;
      except
        Compare := 0;
        Exit;
      end;
    end;
    if TagValue.SortDirection = cLVSD_Ascending then begin
      if SortByData then begin
        Compare := D1 - D2;
      end else begin
        Compare := CompareText(S1, S2);
      end;
    end else begin
      if SortByData then begin
        Compare := D2 - D1;
      end else begin
        Compare := CompareText(S2, S1);
      end;
    end;
  except
    Compare := 0;
    Exit;
  end;
end;

end.
