unit UOptions;

interface

uses
  MyReg, SysUtils, Classes, Contnrs, IniFiles;

type

  { *** TAccountData *** }

  TAccountData = class(TObject)
  private
    FList: TObjectList;
    FChange, FSavePassword: Boolean;
    FTitle, FServer, FUser, FPassword: String;
    FPort: Integer;
    function GetIndex: Integer;
    procedure SetTitle(Value: String);
    procedure SetServer(Value: String);
    procedure SetPort(Value: Integer);
    function GetPortStr: String;
    procedure SetPortStr(Value: String);
    procedure SetUser(Value: String);
    procedure SetPassword(Value: String);
    procedure SetSavePassword(Value: Boolean);
    function Read(ATitle: String; AReg: TMyReg): Boolean;
    function Write(AReg: TMyReg): Boolean;
    function Import(ATitle: String; AIni: TIniFile): Boolean;
    function Export(AIni: TIniFile): Boolean;
  public
    property Index: Integer read GetIndex;
    property Change: Boolean read FChange write FChange;
    property Title: String read FTitle write SetTitle;
    property Server: String read FServer write SetServer;
    property Port: Integer read FPort write SetPort;
    property PortStr: String read GetPortStr write SetPortStr;
    property User: String read FUser write SetUser;
    property Password: String read FPassword write SetPassword;
    property SavePassword: Boolean read FSavePassword write SetSavePassword;
    constructor Create(AList: TObjectList); virtual;
  end;

  { *** TPOP3DAOptions *** }

  TPOP3DAOptions = class(TObject)
  private
    FAccounts: TObjectList;
    function GetChange: Boolean;
    function GetAccountCount: Integer;
    function GetAccount(Index: Integer): TAccountData;
  public
    property Change: Boolean read GetChange;
    property AccountCount: Integer read GetAccountCount;
    property Account[Index: Integer]: TAccountData read GetAccount;
    constructor Create; virtual;
    destructor Destroy; override;
    function AccountAdd: TAccountData;
    procedure AccountDel(Index: Integer);
    procedure AccountClear;
    function Read: Boolean;
    function Write: Boolean;
    function Import(FileName: String): Boolean;
    function Export(FileName: String): Boolean;
  end;

implementation

{ *** TAccountData *** }

function TAccountData.GetIndex: Integer;
begin
  if FList <> nil then Result := FList.IndexOf(Self)
  else Result := -1;
end;

procedure TAccountData.SetTitle(Value: String);
begin
  if FTitle <> Value then begin
    FTitle := Value;
    FChange := True;
  end;
end;

procedure TAccountData.SetServer(Value: String);
begin
  if FServer <> Value then begin
    FServer := Value;
    FChange := True;
  end;
end;

procedure TAccountData.SetPort(Value: Integer);
begin
  if FPort <> Value then begin
    FPort := Value;
    FChange := True;
  end;
end;

function TAccountData.GetPortStr: String;
begin
  Result := IntToStr(FPort);
end;

procedure TAccountData.SetPortStr(Value: String);
begin
  SetPort(StrToIntDef(Value, 110));
end;

procedure TAccountData.SetUser(Value: String);
begin
  if FUser <> Value then begin
    FUser := Value;
    FChange := True;
  end;
end;

procedure TAccountData.SetPassword(Value: String);
begin
  if FPassword <> Value then begin
    FPassword := Value;
    FChange := True;
  end;
end;

procedure TAccountData.SetSavePassword(Value: Boolean);
begin
  if FSavePassword <> Value then begin
    FSavePassword := Value;
    FChange := True;
  end;
end;

function TAccountData.Read(ATitle: String; AReg: TMyReg): Boolean;
begin
  Result := False;
  try
    if AReg.OpenKeyReadOnly(ATitle) then begin
      FTitle := ATitle;
      FServer := AReg.ReadStr('Server', '');
      FPort := AReg.ReadInt('Port', 110);
      FUser := AReg.ReadStr('User', '');
      if AReg.ValueExists('Password') then begin
        FPassword := AReg.ReadStr('Password', '');
        FSavePassword := True;
      end else begin
        FPassword := '';
        FSavePassword := False;
      end;
      FChange := False;
      Result := (Fserver > '') and (FUser > '');
      AReg.OpenKeyReadOnly('..');
    end;
  except
  end;
end;

function TAccountData.Write(AReg: TMyReg): Boolean;
begin
  Result := False;
  try
    if AReg.OpenKey(FTitle, True) then begin
      AReg.WriteStr('Server', FServer);
      AReg.WriteInt('Port', FPort);
      AReg.WriteStr('User', FUser);
      if FSavePassword then AReg.WriteStr('Password', FPassword)
      else if AReg.ValueExists('Password') then AReg.DeleteValue('Password');
      FChange := False;
      Result := True;
      AReg.OpenKey('..', False);
    end;
  except
  end;
end;

function TAccountData.Import(ATitle: String; AIni: TIniFile): Boolean;
begin
  Result := False;
  try
    if AIni.SectionExists(ATitle) then begin
      FTitle := ATitle;
      FServer := AIni.ReadString(ATitle, 'Server', '');
      FPort := AIni.ReadInteger(ATitle, 'Port', 110);
      FUser := AIni.ReadString(ATitle, 'User', '');
      if AIni.ValueExists(ATitle, 'Password') then begin
        FPassword := AIni.ReadString(ATitle, 'Password', '');
        FSavePassword := True;
      end else begin
        FPassword := '';
        FSavePassword := False;
      end;
      FChange := True;
      Result := (Fserver > '') and (FUser > '');
    end;
  except
  end;
end;

function TAccountData.Export(AIni: TIniFile): Boolean;
begin
  Result := False;
  try
    AIni.WriteString(FTitle, 'Server', FServer);
    AIni.WriteInteger(FTitle, 'Port', FPort);
    AIni.WriteString(FTitle, 'User', FUser);
    if FSavePassword then AIni.WriteString(FTitle, 'Password', FPassword)
    else if AIni.ValueExists(FTitle, 'Password') then AIni.DeleteKey(FTitle, 'Password');
    Result := True;
  except
  end;
end;

constructor TAccountData.Create(AList: TObjectList);
begin
  FList := AList;
  FChange := True;
  FTitle := '';
  FServer := '';
  FPort := 110;
  FUser := '';
  FPassword := '';
end;


{ *** TPOP3DAOptions *** }

function TPOP3DAOptions.GetChange: Boolean;
var
  iAccounts: Integer;
begin
  Result := False;
  for iAccounts := 0 to FAccounts.Count - 1 do
    if TAccountData(FAccounts[iAccounts]).FChange then begin
      Result := True;
      Break;
    end;
end;

function TPOP3DAOptions.GetAccountCount: Integer;
begin
  Result := FAccounts.Count;
end;

function TPOP3DAOptions.GetAccount(Index: Integer): TAccountData;
begin
  if (Index >= 0) and (Index < FAccounts.Count) then Result := TAccountData(FAccounts[Index])
  else Result := nil;
end;

constructor TPOP3DAOptions.Create;
begin
  FAccounts := TObjectList.Create(True);
end;

destructor TPOP3DAOptions.Destroy;
begin
  FAccounts.Free;
  Inherited Destroy;
end;

function TPOP3DAOptions.AccountAdd: TAccountData;
begin
  Result := TAccountData.Create(FAccounts);
  FAccounts.Add(Result);
end;

procedure TPOP3DAOptions.AccountDel(Index: Integer);
begin
  if FAccounts.Count > Index then FAccounts.Delete(Index);
end;

procedure TPOP3DAOptions.AccountClear;
begin
  FAccounts.Clear;
end;

function TPOP3DAOptions.Read: Boolean;
var
  Reg: TMyReg;
  AccountTitles: TStrings;
  iAccounts: Integer;
begin
  Result := False;
  FAccounts.Clear;
  try
    AccountTitles := TStringList.Create;
    Reg := TMyReg.Create(REG_CU, 'POP3DA', 'POP3 Direct Access');
    try
      Reg.GetKeyNames(AccountTitles);
      for iAccounts := 0 to AccountTitles.Count - 1 do
        AccountAdd.Read(AccountTitles[iAccounts], Reg);
      Result := True;
    finally
      Reg.Free;
      AccountTitles.Free;
    end;
  except
  end;
end;

function TPOP3DAOptions.Write: Boolean;
var
  Reg: TMyReg;
  AccountTitles: TStrings;
  iAccounts, iAccountTitles: Integer;
begin
  Result := False;
  try
    AccountTitles := TStringList.Create;
    Reg := TMyReg.Create(REG_CU, 'POP3DA', 'POP3 Direct Access');
    try
      Result := True;
      Reg.GetKeyNames(AccountTitles);
      for iAccounts := 0 to FAccounts.Count - 1 do begin
        Result := TAccountData(FAccounts[iAccounts]).Write(Reg) and Result;
        // AccountTitles hilft, die Accounts zu erkennen, die aus den Options gelscht wurden.
        // Diese Accounts mssen dann auch aus der Registry gelscht werden.
        iAccountTitles := AccountTitles.IndexOf(TAccountData(FAccounts[iAccounts]).Title);
        if iAccountTitles >= 0 then AccountTitles.Delete(iAccountTitles);
      end;
      for iAccounts := 0 to AccountTitles.Count - 1 do
        Result := Reg.DeleteKey(AccountTitles[iAccounts]) and Result;
    finally
      Reg.Free;
      AccountTitles.Free;
    end;
  except
  end;
end;

function TPOP3DAOptions.Import(FileName: String): Boolean;
var
  Ini: TIniFile;
  AccountTitles: TStrings;
  iAccounts: Integer;
begin
  Result := False;
  try
    AccountTitles := TStringList.Create;
    Ini := TIniFile.Create(FileName);
    try
      Ini.ReadSections(AccountTitles);
      for iAccounts := 0 to AccountTitles.Count - 1 do
        AccountAdd.Import(AccountTitles[iAccounts], Ini);
      Result := True;
    finally
      Ini.Free;
      AccountTitles.Free;
    end;
  except
  end;
end;

function TPOP3DAOptions.Export(FileName: String): Boolean;
var
  Ini: TIniFile;
  iAccounts: Integer;
begin
  Result := False;
  if FileExists(FileName) then DeleteFile(FileName);
  try
    Ini := TIniFile.Create(FileName);
    try
      Result := True;
      for iAccounts := 0 to FAccounts.Count - 1 do
        Result := TAccountData(FAccounts[iAccounts]).Export(Ini) and Result;
    finally
      Ini.Free;
    end;
  except
  end;
end;


end.
