unit UVerInfo;

interface

uses
  Windows, SysUtils;

type
  TFileVersionDW = record
    MS, LS: Cardinal;
  end;
  TFileVersionNumber = record
    SubVersion, Version, Build, Revision: Word;
  end;
  TFileVersionLang = record
    Language, CodePage: Word;
  end;
  PFileVersionLang = ^TFileVersionLang;
  TFileVersionInformation = class(TObject)
  private
    FFileName: String;
    FFixedInfo: TVSFixedFileInfo;
    FVIBuffer: PByte;
    FVIBufferLen: Cardinal;
    FVILangs: PFileVersionLang;
    FVILangsCount: Cardinal;
    FDefaultLang: TFileVersionLang;
    function HasdwFileFlags(Flag: Word): Boolean; virtual;
  protected
    function FLanguagesCount: Cardinal; virtual;
    function FLanguages(Index: Cardinal): TFileVersionLang; virtual;
    function FFileVersion: TFileVersionNumber; virtual;
    function FProductVersion: TFileVersionNumber; virtual;
    function FCompanyName: String; virtual;
    function FFileDescription: String; virtual;
    function FInternalName: String; virtual;
    function FLegalCopyRight: String; virtual;
    function FLegalTradeMarks: String; virtual;
    function FOriginalFileName: String; virtual;
    function FProductName: String; virtual;
    function FComments: String; virtual;
    function FSpecialBuild: String; virtual;
    function FPrivateBuild: String; virtual;
    function FIsDebugBuild: Boolean; virtual;
    function FIsPreRelease: Boolean; virtual;
    function FIsPrivateBuild: Boolean; virtual;
    function FIsSpecialBuild: Boolean; virtual;
    function FIsPatched: Boolean; virtual;
    function FIsInfoInferred: Boolean; virtual;
  public
    property FileName: String read FFileName;
    property FixedInfo: TVSFixedFileInfo read FFixedInfo;
    property LanguagesCount: Cardinal read FLanguagesCount;
    property Languages[Index: Cardinal]: TFileVersionLang read FLanguages;
    property DefaultLang: TFileVersionLang read FDefaultLang write FDefaultLang;
    property FileVersion: TFileVersionNumber read FFileVersion;
    property ProductVersion: TFileVersionNumber read FProductVersion;
    property CompanyName: String read FCompanyName;
    property FileDescription: String read FFileDescription;
    property InternalName: String read FInternalName;
    property LegalCopyRight: String read FLegalCopyRight;
    property LegalTradeMarks: String read FLegalTradeMarks;
    property OriginalFileName: String read FOriginalFileName;
    property ProductName: String read FProductName;
    property Comments: String read FComments;
    property SpecialBuild: String read FSpecialBuild;
    property PrivateBuild: String read FPrivateBuild;
    property IsDebugBuild: Boolean read FIsDebugBuild;
    property IsPreRelease: Boolean read FIsPreRelease;
    property IsPrivateBuild: Boolean read FIsPrivateBuild;
    property IsSpecialBuild: Boolean read FIsSpecialBuild;
    property IsPatched: Boolean read FIsPatched;
    property IsInfoInferred: Boolean read FIsInfoInferred;
    constructor Create; overload; virtual;
    constructor Create(AFileName: String); overload; virtual;
    procedure Read(AFileName: String); virtual;
    function QueryValue(Lang, Name: String): String; overload; virtual;
    function QueryValue(Lang: TFileVersionLang; Name: String): String; overload; virtual;
    function QueryValue(Name: String): String; overload; virtual;
  end;

implementation

function TFileVersionInformation.FLanguagesCount: Cardinal;
begin
  if FVILangs <> nil then begin
    Result := FVILangsCount;
  end else Result := 0;
end;
function TFileVersionInformation.FLanguages(Index: Cardinal): TFileVersionLang;
begin
  if (FVILangs <> nil) and (FVILangsCount > Index) then begin
    Result := PFileVersionLang(Cardinal(FVILangs) + (Index * 4))^;
  end else begin
    Result.Language := 0;
    Result.CodePage := 0;
  end;
end;

function TFileVersionInformation.FFileVersion: TFileVersionNumber;
var
  Version: TFileVersionDW;
begin
  Version.MS := FFixedInfo.dwFileVersionMS;
  Version.LS := FFixedInfo.dwFileVersionLS;
  Result := TFileVersionNumber(Version);
end;

function TFileVersionInformation.FProductVersion: TFileVersionNumber;
var
  Version: TFileVersionDW;
begin
  Version.MS := FFixedInfo.dwProductVersionMS;
  Version.LS := FFixedInfo.dwProductVersionLS;
  Result := TFileVersionNumber(Version);
end;

function TFileVersionInformation.FCompanyName: String;
begin
  Result := QueryValue('CompanyName');
end;
function TFileVersionInformation.FFileDescription: String;
begin
  Result := QueryValue('FileDescription');
end;
function TFileVersionInformation.FInternalName: String;
begin
  Result := QueryValue('InternalName');
end;
function TFileVersionInformation.FLegalCopyRight: String;
begin
  Result := QueryValue('LegalCopyRight');
end;
function TFileVersionInformation.FLegalTradeMarks: String;
begin
  Result := QueryValue('LegalTradeMarks');
end;
function TFileVersionInformation.FOriginalFileName: String;
begin
  Result := QueryValue('OriginalFileName');
end;
function TFileVersionInformation.FProductName: String;
begin
  Result := QueryValue('ProductName');
end;
function TFileVersionInformation.FComments: String;
begin
  Result := QueryValue('Comments');
end;
function TFileVersionInformation.FSpecialBuild: String;
begin
  Result := QueryValue('SpecialBuild');
end;
function TFileVersionInformation.FPrivateBuild: String;
begin
  Result := QueryValue('PrivateBuild');
end;

function TFileVersionInformation.HasdwFileFlags(Flag: Word): Boolean;
begin
  if (FVIBuffer <> nil) then begin
    Result := (FFixedInfo.dwFileFlagsMask and FFixedInfo.dwFileFlags and Flag) = Flag;
  end else begin
    Result := False;
  end;
end;
function TFileVersionInformation.FIsDebugBuild: Boolean;
begin
  Result := HasdwFileFlags(VS_FF_DEBUG);
end;
function TFileVersionInformation.FIsPreRelease: Boolean;
begin
  Result := HasdwFileFlags(VS_FF_PRERELEASE);
end;
function TFileVersionInformation.FIsPrivateBuild: Boolean;
begin
  Result := HasdwFileFlags(VS_FF_PRIVATEBUILD);
end;
function TFileVersionInformation.FIsSpecialBuild: Boolean;
begin
  Result := HasdwFileFlags(VS_FF_SPECIALBUILD);
end;
function TFileVersionInformation.FIsPatched: Boolean;
begin
  Result := HasdwFileFlags(VS_FF_PATCHED);
end;
function TFileVersionInformation.FIsInfoInferred: Boolean;
begin
  Result := HasdwFileFlags(VS_FF_INFOINFERRED);
end;

constructor TFileVersionInformation.Create;
begin
  FFileName := '';
  FillChar(FFixedInfo, SizeOf(TVSFixedFileInfo), #0);
  FVIBuffer := nil;
  FVIBufferLen := 0;
end;

constructor TFileVersionInformation.Create(AFileName: String);
begin
  Create;
  Read(AFileName);
end;

procedure TFileVersionInformation.Read(AFileName: String);
var
  Reserved, VIFixedLen: Cardinal;
  VIFixed: PVSFixedFileInfo;
begin
  if (FVIBuffer <> nil) then begin
    FreeMem(FVIBuffer);
    FVIBuffer := nil;
    FVILangs := nil;
    FillChar(FFixedInfo, SizeOf(TVSFixedFileInfo), #0);
  end;
  FFileName := '';
  FVIBufferLen := GetFileVersionInfoSize(PChar(AFileName), Reserved);
  if (FVIBufferLen > 0) then begin
    try
      FVIBuffer := AllocMem(FVIBufferLen);
      if GetFileVersionInfo(PChar(AFileName), Reserved, FVIBufferLen, FVIBuffer) then begin
        if VerQueryValue(FVIBuffer, '\', Pointer(VIFixed), VIFixedLen) then begin
          FFixedInfo := VIFixed^;
          FFileName := AFileName;
          if VerQueryValue(FVIBuffer, '\VarFileInfo\Translation', Pointer(FVILangs), FVILangsCount) then begin
            FVILangsCount := FVILangsCount div 4;
            FDefaultLang := Languages[0];
          end else begin
            FVILangs := nil;
          end;
        end;
      end else begin
        if (FVIBuffer <> nil) then begin
          FreeMem(FVIBuffer);
          FVIBuffer := nil;
        end;
        FVIBufferLen := 0;
      end;
    except
      on EOutOfMemory do begin
        FVIBufferLen := 0;
      end;
    end;
  end;
end;

function TFileVersionInformation.QueryValue(Lang, Name: String): String;
var
  VIValue: PChar;
  VIValueLen: Cardinal;
  SubBlock: String;
begin
  Result := '';
  if (FVIBuffer <> nil) then begin
    SubBlock := '\StringFileInfo\' + Lang + '\' + Name;
    if (VerQueryValue(FVIBuffer, PChar(SubBlock), Pointer(VIValue), VIValueLen)) then begin
      Result := Copy(String(VIValue), 1, VIValueLen);
    end;
  end;
end;
function TFileVersionInformation.QueryValue(Lang: TFileVersionLang; Name: String): String;
begin
  Result := QueryValue(Format('%.4x%.4x', [Lang.Language, Lang.CodePage]), Name);
end;
function TFileVersionInformation.QueryValue(Name: String): String;
begin
  Result := QueryValue(FDefaultLang, Name);
end;

end.
